# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_cluster) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    Allows you to manage a WebLogic cluster in a WebLogic domain.

    Here is an example on how you should use this:

        wls_cluster { 'WlsCluster':
          ensure           => 'present',
          messagingmode    => 'unicast',
          migrationbasis   => 'consensus',
          servers          => ['wlsServer3','wlsServer4'],
          multicastaddress => '239.192.0.0',
          multicastport    => '7001',
        }

    In this example you are managing a server in the default domain. When you want to
    manage a cluster in a specific domain, you can use:

        wls_cluster { 'MuDomain/WlsCluster':
          ensure           => 'present',
          messagingmode    => 'unicast',
          migrationbasis   => 'consensus',
          servers          => ['wlsServer3','wlsServer4'],
          multicastaddress => '239.192.0.0',
          multicastport    => '7001',
        }

    Check [the Oracle Weblogic documentation](http://docs.oracle.com/middleware/1221/wls/INTRO/clustering.htm#INTRO185)
    for more documentation about setting up and managing a WebLogic cluster.

    DESC

    ensurable

    #
    # Only use elements on the path where this condition is true. The condition is
    # that is has specified serverTemplates
    #
    index_condition do |name, path, wls_config|
      #
      # templates doe not seem te be available on WebLogic 10.3.6. So allow for an error
      # and return and empty array.
      #
      templates = wls_config.children("#{path}/#{name}/DynamicServers/#{name}/ServerTemplate/") { [] }
      templates.empty?
    end

    bean_type('Cluster')

    parameter :domain
    parameter :name
    parameter :cluster_name
    parameter :timeout
    property :servers
    property :clusteraddress
    property :migrationbasis
    property :migration_datasource
    property :migration_table_name
    property :messagingmode
    property :datasourceforjobscheduler
    property :unicastbroadcastchannel
    property :multicastaddress
    property :multicastport
    property :frontendhost
    property :frontendhttpport
    property :frontendhttpsport
    property :securereplication
    property :candidate_machines_for_migratable_servers

    extraproperties do
      property :client_cert_proxy_enabled
      property :cluster_broadcast_channel
      property :cluster_type
      property :fencing_grace_period_millis
      property :health_check_periods_until_fencing
      property :job_scheduler_table_name
      property :migration_basis
      property :number_of_servers_in_cluster_address
      property :persist_sessions_on_shutdown
      property :session_flush_interval
      property :cluster_address
      property :database_leasing_basis_connection_retry_count
      property :data_source_for_automatic_migration
      property :health_check_interval_millis
      property :idle_periods_until_timeout
      property :inter_cluster_comm_link_health_check_interval
      property :message_ordering_enabled
      property :millis_to_sleep_between_auto_migration_attempts
      property :multicast_address
      property :multicast_ttl
      property :secure_replication_enabled
      property :additional_auto_migration_attempts
      property :auto_migration_table_name
      property :cluster_messaging_mode
      property :database_leasing_basis_connection_retry_delay
      property :data_source_for_job_scheduler
      property :data_source_for_session_persistence
      property :default_load_algorithm
      property :frontend_host
      property :frontend_http_port
      property :frontend_https_port
      property :http_trace_support_enabled
      property :member_death_detector_enabled
      property :member_warmup_timeout_seconds
      property :multicast_buffer_size
      property :multicast_data_encryption
      property :multicast_send_delay
      property :notes
      property :remote_cluster_address
      property :replication_channel
      property :session_flush_threshold
      property :txn_affinity_enabled
      property :weblogic_plugin_enabled
      property :async_session_queue_timeout
      property :greedy_session_flush_interval
      property :multicast_port
      property :one_way_rmi_for_replication_enabled
      property :replication_timeout_enabled
      property :service_age_threshold_seconds
      property :session_lazy_deserialization_enabled
      property :wan_session_persistence_table_name
      #
      # Weblogic 12.2.1 new properties
      #
      property :service_activation_request_response_timeout
      property :session_state_query_protocol_enabled
      property :session_state_query_request_timeout
      property :tags
      #
      # WebLogic 12.2 JTA properties
      #
      property :jta_cross_site_recovery_retry_interval
      property :jta_max_resource_unavailable_millis
      property :jta_max_transactions
      property :jta_wsat_transport_security_mode
      property :jta_security_interop_mode
      property :jta_max_retry_seconds_before_determiner_fail
      property :jta_abandon_timeout_seconds
      property :jta_purge_resource_from_checkpoint_interval_seconds
      property :jta_tlog_write_when_determiner_exists_enabled
      property :jta_timeout_seconds
      property :jta_forget_heuristics
      property :jta_parallel_xa_enabled
      property :jta_two_phase_enabled
      property :jta_shutdown_grace_period
      property :jta_recovery_site_name
      property :jta_serialize_enlistments_gc_interval_millis
      property :jta_wsat_issued_token_enabled
      property :jta_checkpoint_interval_seconds
      property :jta_cross_site_recovery_lease_update
      property :jta_notes
      property :jta_tags
      property :jta_before_completion_iteration_limit
      property :jta_unregister_resource_grace_period
      property :jta_tightly_coupled_transactions_enabled
      property :jta_recovery_threshold_millis
      property :jta_determiners
      property :jta_max_xa_call_millis
      property :jta_determiner_candidate_resource_info_list
      property :jta_max_resource_requests_on_server
      property :jta_max_unique_name_statistics
      property :jta_cross_site_recovery_lease_expiration
      property :jta_migration_checkpoint_interval_seconds
      property :jta_cross_domain_recovery_retry_interval
      property :jta_max_transactions_health_interval_millis
      property :jta_parallel_xa_dispatch_policy
      property :jta_completion_timeout_seconds
      property :jta_clusterwide_recovery_enabled
      #
      # overload protection properties
      #
      property :overload_panic_action
      property :overload_shared_capacity_for_work_managers
      property :overload_failure_action
      property :overload_server_failure_trigger
      property :overload_free_memory_percent_low_threshold
      property :overload_free_memory_percent_high_threshold
      property :overload_notes
    end

    add_title_attributes(:cluster_name)

    #
    # Manage auto requires
    #
    child_of(:wls_datasource) do
      ["#{domain}/#{migration_datasource}",
       "#{domain}/#{datasourceforjobscheduler}"]
    end
  end
end
