# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_foreign_server_object) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    This resource allows you to manage a foreign server object in a JMS Module of a WebLogic domain.

    Here is an example on how you should use this:

        wls_foreign_server_object { 'jmsClusterModule:Jboss:CF':
          ensure         => 'present',
          localjndiname  => 'jms/jboss/CF',
          object_type    => 'connectionfactory',
          remotejndiname => 'jms/Remote/CF',
        }

    In this example you are managing a foreign server object in the default domain. When you want to
    manage a foreign server object in a specific domain, you can use:

    wls_foreign_server_object { 'my_domain/jmsClusterModule:Jboss:JBossQ':
      ensure         => 'present',
      localjndiname  => 'jms/jboss/Queue',
      object_type    => 'destination',
      remotejndiname => 'jms/Remote/Queue',
    }

    DESC

    to_get_raw_resources do
      enable_extended_methods
      destinations = configuration.keys.collect do |domain|
        path = Puppet_X::EnterpriseModules::WebLogic::BeanPath.new("#{bean_path}/ForeignDestinations")
        resources_at(domain, path, {}, [])
      end.flatten
      connection_factories = configuration.keys.collect do |domain|
        path = Puppet_X::EnterpriseModules::WebLogic::BeanPath.new("#{bean_path}/ForeignConnectionFactories")
        resources_at(domain, path, {}, [])
      end.flatten
      destinations.collect { |e| e.merge!(:object_type => :destination) } + connection_factories.collect { |e| e.merge!(:object_type => :connectionfactory) }
    end

    def on_create
      Puppet.info "create #{name} "
      actual_creation_path = bean_path.subst(to_hash)
      fail "Parent resource #{actual_creation_path} doesn't exists." unless wls_config.exists?(actual_creation_path)

      wls_config.create(bean_type, actual_creation_path, self[key_name])
      nil
    end

    def on_destroy
      Puppet.info "destroy #{name} "
      full_path = "#{bean_path.subst(to_hash)}/#{extention_path}"
      wls_config.destroy(bean_type, full_path, self[key_name])
    end

    bean_path('/JMSSystemResources/%<jmsmodule>s/JMSResource/%<jmsmodule>s/ForeignServers/%<foreign_server>s')

    ensurable
    parameter :domain
    parameter :name
    parameter :object_name
    parameter :jmsmodule
    parameter :foreign_server
    parameter :timeout
    property  :object_type # This one is unmodifiable though

    property :remotejndiname
    property :localjndiname

    extraproperties do
      property :connection_health_checking
      property :local_jndi_name
      property :password
      property :username
      property :notes
      property :remote_jndi_name
      #
      # Weblogic 12.2.1 new properties
      #
    end

    add_title_attributes(:jmsmodule, :foreign_server, :object_name) do
      /(.+):(.+):(.+)/
    end

    private

    def extention_path
      case self[:object_type].to_s
      when 'destination'
        'ForeignDestinations'
      when 'connectionfactory'
        'ForeignConnectionFactories'
      else
        fail "#{self[:object_type]} is not valid as an object_type. internal error"
      end
    end

    def bean_type
      case self[:object_type].to_s
      when 'destination'
        'ForeignDestination'
      when 'connectionfactory'
        'ForeignConnectionFactory'
      else
        fail "#{self[:object_type]} is not valid as an object_type. internal error"
      end
    end
  end
end
