# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'
require "#{File.dirname(__FILE__)}/wls_saf_imported_destination_object/saf_imported_destination_object_property"

module Puppet
  Type.newtype(:wls_saf_imported_destination_object) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    This resource allows you to manage a SAF imported destinations object in a JMS Module of a WebLogic domain.

    Here is an example on how you should use this:

        wls_saf_imported_destination_object { 'jmsClusterModule:SAFImportedDestinations-0:SAFDemoQueue':
          ensure               => 'present',
          nonpersistentqos     => 'Exactly-Once',
          object_type          => 'queue',
          remotejndiname       => 'jms/DemoQueue',
          unitoforderrouting   => 'Hash',
          usetimetolivedefault => '0',
        }

    In this example you are managing a SAF imported destination object in the default domain. When you want to
    manage a SAF imported destination object in a specific domain, you can use:

        wls_saf_imported_destination_object { 'my_domain/jmsClusterModule:SAFImportedDestinations-0:SAFDemoTopic':
          ensure               => 'present',
          nonpersistentqos     => 'Exactly-Once',
          object_type          => 'topic',
          remotejndiname       => 'jms/DemoTopic',
          timetolivedefault    => '100000000',
          unitoforderrouting   => 'Hash',
          usetimetolivedefault => 1,
        }

    DESC

    to_get_raw_resources do
      enable_extended_methods
      queues = configuration.keys.collect do |domain|
        path = Puppet_X::EnterpriseModules::WebLogic::BeanPath.new("#{bean_path}/SAFQueues")
        resources_at(domain, path, {}, [])
      end.flatten
      topics = configuration.keys.collect do |domain|
        path = Puppet_X::EnterpriseModules::WebLogic::BeanPath.new("#{bean_path}/SAFTopics")
        resources_at(domain, path, {}, [])
      end.flatten
      queues.collect { |e| e.merge!(:object_type => :queue) } + topics.collect { |e| e.merge!(:object_type => :topic) }
    end

    def on_create
      Puppet.info "create #{name} "
      actual_creation_path = bean_path.subst(to_hash)
      fail "Parent resource #{actual_creation_path} doesn't exists." unless wls_config.exists?(actual_creation_path)

      wls_config.create(bean_type, actual_creation_path, self[key_name])
      nil
    end

    def on_destroy
      Puppet.info "destroy #{name} "
      full_path = "#{bean_path.subst(to_hash)}/#{extention_path}"
      wls_config.destroy(bean_type, full_path, self[key_name])
    end

    bean_path('/JMSSystemResources/%<jmsmodule>s/JMSResource/%<jmsmodule>s/SAFImportedDestinations/%<imported_destination>s')

    ensurable
    parameter :object_name
    parameter :imported_destination
    parameter :domain
    parameter :name
    parameter :jmsmodule
    parameter :timeout

    property :object_type
    property :timetolivedefault
    property :usetimetolivedefault
    property :unitoforderrouting
    property :remotejndiname
    property :localjndiname
    property :nonpersistentqos
    extraproperties do
      property :local_jndi_name
      property :log_message_logging_enabled
      property :log_message_logging_format
      property :log_template_bean
      property :non_persistent_qos
      property :saf_error_handling
      property :notes
      property :persistent_qos
      property :remote_jndi_name
      property :time_to_live_default
      property :unit_of_order_routing
      property :use_saf_time_to_live_default
      #
      # Weblogic 12.2.1 new properties
      #
    end

    add_title_attributes(:jmsmodule, :imported_destination, :object_name) do
      /(.+):(.+):(.+)/
    end

    private

    def extention_path
      case self[:object_type].to_s
      when 'queue'
        'SAFQueues'
      when 'topic'
        'SAFTopics'
      else
        fail "#{self[:object_type]} is not valid as an object_type. internal error"
      end
    end

    def bean_type
      case self[:object_type].to_s
      when 'queue'
        'SAFQueue'
      when 'topic'
        'SAFTopic'
      else
        fail "#{self[:object_type]} is not valid as an object_type. internal error"
      end
    end
  end
end
