# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:password_reset) do
  include EasyType

  # rubocop: disable Lint/BooleanSymbol
  defaultto do
    !ENV['WLS_USER_PASSWORD_RESET'].nil? ? :true : :false
  end
  # rubocop: enable Lint/BooleanSymbol

  desc <<-DESC
  reset the password for the WebLogic user.

  **Requires wls_config v3.5.2 or higher**

  In normal cases, the password is used just as a parameter. This means that even
  when the password in your puppet code is different, Puppet will not manage it. This
  is because WebLogic does not expose the (encrypted) password, so Puppet cannot manage it.

  For some use cases, however, it is required that puppet sets (or reset's) the password
  to a specified value. For these use cases, you can use the property `password_reset`.

  There are three ways to use this:

    1) Set the value of password_reset to the boolean value true

    In this case, the value set as the password will be used to reset the current
    password.

    An example on how to use this:

        wls_user{test_user:
          ensure         => 'present',
          password       => 'verysecret1!',
          password_reset => true,
        }

    2) Set the value of password_reset to a string representing the requested new password

    In this case, the value of `password_reset` will be used to reset the current
    password.

    An example on how to use this:

        wls_user{test_user:
          ensure         => 'present',
          password_reset => 'verysecret1!',
        }

    3) Set the environment variable WLS_USER_PASSWORD_RESET to true

    In this case the value set as password will be used to reset the current
    password. No need to add the property `password_reset` to your manifest

    An example on how to use this:

        $ export WLS_USER_PASSWORD_RESET=true


        wls_user{test_user:
          ensure         => 'present',
          password       => 'verysecret1!',
        }
  #{'  '}
    The environment variable must be set on the agent (or location where you use`$ puppet apply`). This
    use case helps when you want to reset **ALL** passwords currently set in the puppet manifests, But
    without changing any of the manifest code.
  DESC

  def insync?(_)
    should.to_s == 'false'
  end

  def change_to_s(_currentvalue, _newvalue)
    'password reset to requested value'
  end

  to_translate_to_resource do |_raw_resource|
    nil
  end

  data_type('Optional[Variant[Boolean,String[1]]]')
end
